@@ -17,13 +17,15 @@
 # ISOFIT: Imaging Spectrometer Optimal FITting
 # Author: Philip G. Brodrick, philip.brodrick@jpl.nasa.gov
 
-from typing import Dict, List, Type
-from isofit.configs.base_config import BaseConfigSection
-from isofit.configs.sections.statevector_config import StateVectorConfig
-import numpy as np
+import logging
 import os
 from collections import OrderedDict
-import logging
+from typing import Dict, List, Type
+
+import numpy as np
+
+from isofit.configs.base_config import BaseConfigSection
+from isofit.configs.sections.statevector_config import StateVectorConfig
 
 
 class RadiativeTransferEngineConfig(BaseConfigSection):
@@ -32,18 +34,23 @@ class RadiativeTransferEngineConfig(BaseConfigSection):
     """
 
     def __init__(self, sub_configdic: dict = None, name: str = None):
+        super().__init__()
         self._name_type = str
         self.name = name
         """str: Name of config - optional, and not currently used."""
 
         self._engine_name_type = str
         self.engine_name = None
-        """str: Name of radiative transfer engine to use - options ['modtran', 'libradtran', '6s']."""
+        """str: Name of radiative transfer engine to use - options ['modtran', '6s', 'sRTMnet']."""
 
         self._engine_base_dir_type = str
         self.engine_base_dir = None
         """str: base directory of the given radiative transfer engine on user's OS."""
 
+        self._engine_lut_file_type = str
+        self.engine_lut_file = None
+        """str: File containing the prebuilt LUT file (hdf5)."""
+
         self._wavelength_file_type = str
         self.wavelength_file = None
         """str: Optional path to wavelength file for high-res atmospheric calculations"""
@@ -60,13 +67,40 @@ class RadiativeTransferEngineConfig(BaseConfigSection):
         self.lut_path = None
         """str: The path to the look up table directory used by the radiative transfer engine."""
 
+        self._sim_path_type = str
+        self.sim_path = None
+        """str: Path to the simulation outputs for the radiative transfer engine."""
+
         self._template_file_type = str
         self.template_file = None
         """str: A template file to be used as the base-configuration for the given radiative transfer engine."""
 
-        self._lut_names_type = list()
+        self._treat_as_emissive_type = bool
+        self.treat_as_emissive = False
+        """bool: Run the simulation in emission mode"""
+
+        self._topography_model_type = bool
+        self.topography_model = False
+        """
+        Flag to indicated whether to use a topographic-flux (topoflux)
+        implementation of the forward model.
+        """
+
+        self._glint_model_type = bool
+        self.glint_model = False
+        """
+        Flag to indicate whether to use the sun and sky glint model from Gege (2012, 2014) in the forward model.
+        Only currently functional with multipart MODTRAN.
+        """
+
+        self._rt_mode_type = str
+        self.rt_mode = None
+        """str: Radiative transfer mode of LUT simulations.
+        'transm' for transmittances, 'rdn' for reflected radiance."""
+
+        self._lut_names_type = dict()
         self.lut_names = None
-        """List: Names of the elements to run this radiative transfer element on.  Must be a subset
+        """Dictionary: Names of the elements to run this radiative transfer element on.  Must be a subset
         of the keys in radiative_transfer->lut_grid.  If not specified, uses all keys from
         radiative_transfer-> lut_grid.  Auto-sorted (alphabetically) below."""
 
@@ -149,18 +183,28 @@ class RadiativeTransferEngineConfig(BaseConfigSection):
         self.irradiance_file = None
         """str: 6s-only irradiance file."""
 
+        # MODTRAN and 6S
+        self._rte_configure_and_exit_type = bool
+        self.rte_configure_and_exit = False
+        """bool: Indicates that code should terminate as soon as all radiative transfer engine configuration files are
+        written (without running them)"""
+
         self.set_config_options(sub_configdic)
 
         if self.lut_names is not None:
-            self.lut_names.sort()
+            keys = list(self.lut_names.keys())
+            keys.sort()
+            self.lut_names = {i: self.lut_names[i] for i in keys}
 
         if self.statevector_names is not None:
             self.statevector_names.sort()
 
         if self.interpolator_base_path is None and self.emulator_file is not None:
-            self.interpolator_base_path = self.emulator_file + '_interpolator'
-            logging.info('No interpolator base path set, and emulator used, so auto-setting interpolator path at: {}'.format(self.interpolator_base_path))
-
+            self.interpolator_base_path = self.emulator_file + "_interpolator"
+            logging.info(
+                "No interpolator base path set, and emulator used, so auto-setting"
+                " interpolator path at: {}".format(self.interpolator_base_path)
+            )
 
     def _check_config_validity(self) -> List[str]:
         errors = list()
@@ -168,37 +212,64 @@ class RadiativeTransferEngineConfig(BaseConfigSection):
         # Check that all input files exist
         for key in self._get_nontype_attributes():
             value = getattr(self, key)
-            if value is not None and key[-5:] == '_file' and key != 'emulator_file':
+            if value is not None and key[-5:] == "_file" and key != "emulator_file":
                 if os.path.isfile(value) is False:
-                    errors.append('Config value radiative_transfer->{}: {} not found'.format(key, value))
+                    errors.append(
+                        "Config value radiative_transfer->{}: {} not found".format(
+                            key, value
+                        )
+                    )
 
-        valid_rt_engines = ['modtran', 'libradtran', '6s', 'sRTMnet']
+        valid_rt_engines = ["modtran", "6s", "sRTMnet", "KernelFlowsGP"]
         if self.engine_name not in valid_rt_engines:
-            errors.append('radiative_transfer->raditive_transfer_model: {} not in one of the available models: {}'.
-                          format(self.engine_name, valid_rt_engines))
+            errors.append(
+                "radiative_transfer->raditive_transfer_model: {} not in one of the"
+                " available models: {}".format(self.engine_name, valid_rt_engines)
+            )
 
-        if self.multipart_transmittance and self.engine_name != 'modtran':
-            errors.append('Multipart transmittance is supported for MODTRAN only')
+        valid_rt_modes = ["transm", "rdn"]
+        if self.rt_mode not in valid_rt_modes:
+            errors.append(
+                "radiative_transfer->raditive_transfer_mode: {} not in one of the"
+                " available modes: {}".format(self.rt_mode, valid_rt_modes)
+            )
 
-        if self.earth_sun_distance_file is None and self.engine_name == '6s':
-            errors.append('6s requires earth_sun_distance_file to be specified')
+        if self.multipart_transmittance and self.engine_name != "modtran":
+            errors.append("Multipart transmittance is supported for MODTRAN only")
 
-        if self.irradiance_file is None and self.engine_name == '6s':
-            errors.append('6s requires irradiance_file to be specified')
+        if self.earth_sun_distance_file is None and self.engine_name == "6s":
+            errors.append("6s requires earth_sun_distance_file to be specified")
 
-        if self.engine_name == 'sRTMnet' and self.emulator_file is None:
-            errors.append('The sRTMnet requires an emulator_file to be specified.')
+        if self.irradiance_file is None and self.engine_name == "6s":
+            errors.append("6s requires irradiance_file to be specified")
 
-        if self.engine_name == 'sRTMnet' and self.emulator_aux_file is None:
-            errors.append('The sRTMnet requires an emulator_aux_file to be specified.')
+        if self.engine_name == "sRTMnet" and self.emulator_file is None:
+            errors.append("The sRTMnet requires an emulator_file to be specified.")
 
-        files = [self.earth_sun_distance_file, self.irradiance_file,
-                 self.obs_file, self.aerosol_model_file, self.aerosol_template_file]
+        if self.engine_name == "sRTMnet" and self.emulator_aux_file is None:
+            errors.append("The sRTMnet requires an emulator_aux_file to be specified.")
+
+        if self.engine_name == "sRTMnet" and self.emulator_file is not None:
+            if os.path.splitext(self.emulator_file)[1] != ".h5":
+                errors.append(
+                    "sRTMnet now requires the emulator_file to be of type .h5.  Please download an updated version from:\n https://zenodo.org/records/10831425"
+                )
+
+        files = [
+            self.earth_sun_distance_file,
+            self.irradiance_file,
+            self.obs_file,
+            self.aerosol_model_file,
+            self.aerosol_template_file,
+        ]
         for f in files:
             if f is not None:
                 if os.path.isfile(f) is False:
-                    errors.append('Radiative transfer engine file not found on system: {}'.
-                                  format(self.earth_sun_distance_file))
+                    errors.append(
+                        "Radiative transfer engine file not found on system: {}".format(
+                            self.earth_sun_distance_file
+                        )
+                    )
         return errors
 
 
@@ -208,6 +279,7 @@ class RadiativeTransferUnknownsConfig(BaseConfigSection):
     """
 
     def __init__(self, sub_configdic: dict = None):
+        super().__init__()
         self._H2O_ABSCO_type = float
         self.H2O_ABSCO = None
 
@@ -225,15 +297,6 @@ class RadiativeTransferConfig(BaseConfigSection):
     """
 
     def __init__(self, sub_configdic: dict = None):
-
-        self._topography_model_type = bool
-        self.topography_model = False
-        """
-        Flag to indicated whether to use atopographic-flux (topoflux)
-        implementation of the forward model. Only currently functional
-        with multipart MODTRAN
-        """
-
         self._statevector_type = StateVectorConfig
         self.statevector: StateVectorConfig = StateVectorConfig({})
 
@@ -244,14 +307,13 @@ class RadiativeTransferConfig(BaseConfigSection):
         self.unknowns: RadiativeTransferUnknownsConfig = None
 
         self._interpolator_style_type = str
-        self.interpolator_style = 'mlg'
-        """str: Style of interpolation. This argument is in the format [type][-k], eg:
-        - rg    = RegularGrid
-        - nds-k = NDSplines with K degrees
+        self.interpolator_style = "mlg"
+        """str: Style of interpolation.
         - mlg   = Multilinear Grid
+        - rg    = RegularGrid
         Speed performance:
-            mlg >> stacked rg > unstacked nds > stacked nds >> unstacked rg
-        Caching provides significant gains for rg and nds, marginal for mlg"""
+            mlg >> stacked rg >> unstacked rg
+        Caching provides significant gains for rg, marginal for mlg"""
 
         self._overwrite_interpolator_type = bool
         self.overwrite_interpolator = False
@@ -274,7 +336,7 @@ class RadiativeTransferConfig(BaseConfigSection):
         self._radiative_transfer_engines_type = list()
         self.radiative_transfer_engines = []
 
-        self._set_rt_config_options(sub_configdic['radiative_transfer_engines'])
+        self._set_rt_config_options(sub_configdic["radiative_transfer_engines"])
 
     def _set_rt_config_options(self, subconfig):
         if type(subconfig) is list:
@@ -291,32 +353,46 @@ class RadiativeTransferConfig(BaseConfigSection):
 
         for key, item in self.lut_grid.items():
             if len(item) < 2:
-                errors.append('lut_grid item {} has less than the required 2 elements'.format(key))
+                errors.append(
+                    "lut_grid item {} has less than the required 2 elements".format(key)
+                )
+            if np.unique(item).size < len(item):
+                errors.append(f"Detected duplicate values in lut_grid item {key}")
 
         if self.topography_model:
             for rtm in self.radiative_transfer_engines:
-                if rtm.engine_name != 'modtran':
-                    errors.append('All self.forward_model.radiative_transfer.radiative_transfer_engines must ' \
-                                  'be of type "modtran" if forward_model.topograph_model is set to True')
+                if rtm.engine_name != "modtran":
+                    errors.append(
+                        "All self.forward_model.radiative_transfer.radiative_transfer_engines"
+                        ' must be of type "modtran" if forward_model.topograph_model is'
+                        " set to True"
+                    )
                 if rtm.multipart_transmittance is False:
-                    errors.append('All self.forward_model.radiative_transfer.radiative_transfer_engines must ' \
-                                   'have multipart_transmittance set as True if forward_model.topograph_model ' \
-                                   'is set to True')
+                    errors.append(
+                        "All self.forward_model.radiative_transfer.radiative_transfer_engines"
+                        " must have multipart_transmittance set as True if"
+                        " forward_model.topograph_model is set to True"
+                    )
 
         for rte in self.radiative_transfer_engines:
             errors.extend(rte.check_config_validity())
 
-        kinds   = ['rg', 'nds', 'mlg'] # Implemented kinds of interpolator functions
-        kind    = self.interpolator_style[:3]
+        kinds = ["rg", "nds", "mlg"]  # Implemented kinds of interpolator functions
+        kind = self.interpolator_style[:3]
         degrees = self.interpolator_style[4:]
 
         if kind not in kinds:
-            errors.append(f'Interpolator style {self.interpolator_style} must be one of: {kinds}')
-        if kind == 'nds':
+            errors.append(
+                f"Interpolator style {self.interpolator_style} must be one of: {kinds}"
+            )
+        if kind == "nds":
             try:
                 degree = int(degrees)
                 assert degree >= 0 and np.isfinite(degree)
             except:
-                errors.append(f'Invalid degree number. Should be an integer, e.g. nds-3, got {degrees!r} from {self.interpolator_style!r}[4:]')
+                errors.append(
+                    "Invalid degree number. Should be an integer, e.g. nds-3, got"
+                    f" {degrees!r} from {self.interpolator_style!r}[4:]"
+                )
 
         return errors
