@@ -18,18 +18,16 @@
 # Author: David R Thompson, david.r.thompson@jpl.nasa.gov
 #
 
-from argparse import ArgumentError
-import os
 import json
-import xxhash
-import numpy as np
-import scipy.linalg
-from scipy.interpolate import RegularGridInterpolator
+import os
+from collections import OrderedDict
 from os.path import expandvars
 from typing import List
-from collections import OrderedDict
-import ndsplines
 
+import numpy as np
+import scipy.linalg
+import xxhash
+from scipy.interpolate import RegularGridInterpolator
 
 ### Variables ###
 
@@ -39,149 +37,81 @@ eps = 1e-5
 
 ### Classes ###
 
+# Global variable makes it non-shared mem in ray
+Cache = {"stats": {}}
+
+
 class VectorInterpolator:
-    """ Linear look up table interpolator.  Support linear interpolation through radial space by expanding the look
-        up tables with sin and cos dimensions.
+    """Linear look up table interpolator.  Support linear interpolation through radial space by expanding the look
+    up tables with sin and cos dimensions.
 
-        Args:
-            grid_input: list of lists of floats, indicating the gridpoint elements in each grid dimension
-            data_input: n dimensional array of radiative transfer engine outputs (each dimension size corresponds to the
-                        given grid_input list length, with the last dimensions equal to the number of sensor channels)
-            lut_interp_types: a list indicating if each dimension is in radiance (r), degrees (r), or normal (n) units.
-            version: version to use: 'rg' for scipy RegularGridInterpolator, 'nds-k' for ndsplines, where k is the
-                     degrees.
-        """
+    Args:
+        grid_input: list of lists of floats, indicating the gridpoint elements in each grid dimension
+        data_input: n dimensional array of radiative transfer engine outputs (each dimension size corresponds to the
+                    given grid_input list length, with the last dimensions equal to the number of sensor channels)
+        version: version to use: 'rg' for scipy RegularGridInterpolator, 'mlg' for multilinear grid interpolator
+    """
 
-    def __init__(self,
+    def __init__(
+        self,
         grid_input: List[List[float]],
         data_input: np.array,
-        lut_interp_types: List[str],
-        version = 'nds-1'
+        version="mlg",
     ):
-        if version[:3] in ['rg', 'nds']:
+        # Determine if this a singular unique value, if so just return that directly
+        val = data_input[(0,) * data_input.ndim]
+        if np.isnan(val) and np.isnan(data_input).all() or np.all(data_input == val):
+            self.method = -1
+            self.value = val
+            return
+
+        self.single_point_data = None
+
+        # Lists and arrays are mutable, so copy first
+        grid = grid_input.copy()
+        data = data_input.copy()
+
+        # Check if we are using a single grid point. If so, store the grid input.
+        if np.prod(list(map(len, grid))) == 1:
+            self.single_point_data = data
+        self.n = data.shape[-1]
+
+        # RegularGrid
+        if version == "rg":
+            grid_aug = grid + [np.arange(data.shape[-1])]
+            self.itp = RegularGridInterpolator(
+                grid_aug, data, bounds_error=False, fill_value=None
+            )
             self.method = 1
 
-            self.lut_interp_types = lut_interp_types
-            self.single_point_data = None
-
-            # Lists and arrays are mutable, so copy first
-            grid = grid_input.copy()
-            data = data_input.copy()
-
-            # Check if we are using a single grid point. If so, store the grid input.
-            if np.prod(list(map(len, grid))) == 1:
-                self.single_point_data = data
-
-            # expand grid dimensionality as needed
-            [radian_locations] = np.where(self.lut_interp_types == 'r')
-            [degree_locations] = np.where(self.lut_interp_types == 'd')
-            angle_locations    = np.hstack([radian_locations, degree_locations])
-            angle_types        = np.hstack([
-                self.lut_interp_types[radian_locations],
-                self.lut_interp_types[degree_locations]
-            ])
-            for _angle_loc in range(len(angle_locations)):
-                angle_loc = angle_locations[_angle_loc]
-
-                # get original grid at given location
-                original_grid_subset = np.array(grid[angle_loc])
-
-                # convert for angular coordinates
-                if angle_types[_angle_loc] == 'r':
-                    grid_subset_cosin = np.cos(original_grid_subset)
-                    grid_subset_sin   = np.sin(original_grid_subset)
-
-                elif angle_types[_angle_loc] == 'd':
-                    grid_subset_cosin = np.cos(np.deg2rad(original_grid_subset))
-                    grid_subset_sin   = np.sin(np.deg2rad(original_grid_subset))
-
-                # handle the fact that the grid may no longer be in order
-                grid_subset_cosin_order = np.argsort(grid_subset_cosin)
-                grid_subset_sin_order = np.argsort(grid_subset_sin)
-
-                # convert current grid location, and add a second
-                grid[angle_loc] = grid_subset_cosin[grid_subset_cosin_order]
-                grid.insert(angle_loc+1, grid_subset_sin[grid_subset_sin_order])
-
-                # now copy the data to be interpolated through the extra dimension,
-                # at the specific angle_loc axes.  We'll use broadcast_to to do
-                # this, but we need to do it on the last dimension.  So start by
-                # temporarily moving the target axes there, then broadcasting
-                data = np.swapaxes(data, -1, angle_loc)
-                data_dim = list(np.shape(data))
-                data_dim.append(data_dim[-1])
-                data = data[..., np.newaxis] * np.ones(data_dim)
-
-                # Now we need to actually copy the data between the first two axes,
-                # as broadcast_to doesn't do this
-                for ind in range(data.shape[-1]):
-                    data[..., ind] = data[..., :, ind]
-
-                # Now re-order the cosin dimension
-                data = data[..., grid_subset_cosin_order, :]
-                # Now re-order the sin dimension
-                data = data[..., grid_subset_sin_order]
-
-                # now re-arrange the axes so they're in the right order again,
-                dst_axes = np.arange(len(data.shape)-2).tolist()
-                dst_axes.insert(angle_loc, len(data.shape)-2)
-                dst_axes.insert(angle_loc+1, len(data.shape)-1)
-                dst_axes.remove(angle_loc)
-                dst_axes.append(angle_loc)
-                data = np.ascontiguousarray(np.transpose(data, axes=dst_axes))
-
-                # update the rest of the angle locations
-                angle_locations += 1
-
-            self.n = data.shape[-1]
-
-            # RegularGrid
-            if version == 'rg':
-                grid_aug = grid + [np.arange(data.shape[-1])]
-                self.itp = RegularGridInterpolator(grid_aug, data, bounds_error=False, fill_value=None)
-
-            # NDSplines
-            elif version[:3] == 'nds':
-                degrees  = int(version[4:])
-                grid_aug = grid + [np.arange(data.shape[-1]).tolist()]
-                grid_arr = np.stack(np.meshgrid(*grid_aug, indexing='ij'), axis=-1)
-                self.itp = ndsplines.make_interp_spline(grid_arr, data, degrees=degrees)
-
         # Multilinear Grid
-        elif version == 'mlg':
+        elif version == "mlg":
             self.method = 2
 
-            self.gridtuples  = [np.array(t) for t in grid_input]
-            self.gridarrays  = data_input
-            self.binwidth    = [t[1:] - t[:-1] for t in self.gridtuples] # binwidth arrays for each dimension
+            # None to disable, 0 for unlimited, negatives == 1
+            self.cache_size = 1
+
+            self.gridtuples = [np.array(t) for t in grid]
+            self.gridarrays = data
+            self.binwidth = [
+                t[1:] - t[:-1] for t in self.gridtuples
+            ]  # binwidth arrays for each dimension
             self.maxbaseinds = np.array([len(t) - 1 for t in self.gridtuples])
 
         else:
-            raise ArgumentError(None, f'Unknown interpolator version: {version!r}')
+            raise AttributeError(f"Unknown interpolator version: {version!r}")
 
     def _interpolate(self, points):
         """
-        Supports styles 'rg' and 'nds-k'
+        Supports style 'rg'
         """
         # If we only have one point, we can't do any interpolation, so just
         # return the original data.
         if self.single_point_data is not None:
             return self.single_point_data
 
-        x = np.zeros((self.n, len(points) + 1 +
-                      np.sum(self.lut_interp_types != 'n')))
-        offset_count = 0
-        for i in range(len(points)):
-            if self.lut_interp_types[i] == 'n':
-                x[:, i + offset_count] = points[i]
-            elif self.lut_interp_types[i] == 'r':
-                x[:, i + offset_count] = np.cos(points[i])
-                x[:, i + 1 + offset_count] = np.sin(points[i])
-                offset_count += 1
-            elif self.lut_interp_types[i] == 'd':
-                x[:, i + offset_count] = np.cos(np.deg2rad(points[i]))
-                x[:, i + 1 + offset_count] = np.sin(np.deg2rad(points[i]))
-                offset_count += 1
+        x = np.zeros((self.n, len(points) + 1))
+        x[:, :-1] = points
 
         # This last dimension is always an integer so no
         # interpolation is performed. This is done only
@@ -191,9 +121,27 @@ class VectorInterpolator:
 
         return res
 
+    def _lookup(self, i, point):
+        """
+        Calculates the slicing for the cube in _multilinear_grid
+        """
+        j = np.searchsorted(self.gridtuples[i][:-1], point) - 1
+
+        # Bounds functions
+        lower = lambda: max(min(self.maxbaseinds[i], j), 0)
+        upper = lambda: max(min(self.maxbaseinds[i] + 2, j + 2), 2)
+
+        if point >= self.gridtuples[i][-1]:
+            return None, upper() - 1
+        elif point <= self.gridtuples[i][0]:
+            return None, lower()
+        else:
+            delta = (point - self.gridtuples[i][j]) / self.binwidth[i][j]
+            return delta, slice(lower(), upper())
+
     def _multilinear_grid(self, points):
         """
-        Jouni's implementation
+        Cached version of Jouni's implementation
 
         Args:
             points: The point being interpolated. If at the limit, the extremal value in
@@ -202,36 +150,37 @@ class VectorInterpolator:
         Returns:
             cube: np.ndarray
         """
-        # Index of left side of bin, and don't go over (that's why it's t[:-1] instead of t)
-        inds = [
-            np.searchsorted(t[:-1], points[i]) - 1
-            for i, t in enumerate(self.gridtuples)
-        ]
-        deltas = np.array([
-            (points[i] - self.gridtuples[i][j]) / self.binwidth[i][j]
-            for i, j in enumerate(inds)
-        ])
-        diff = 1 - deltas
+        deltas = [None] * points.size
+        idxs = [None] * points.size
 
-        # Set the 'cube' data to be our interpolation data
-        idx = tuple([
-            slice(
-                max(min(self.maxbaseinds[j]  , i  ), 0),
-                max(min(self.maxbaseinds[j]+2, i+2), 2)
-            ) for j, i in enumerate(inds)
-        ])
-        cube = np.copy(self.gridarrays[idx], order='A')
+        for i, point in enumerate(points):
+            if self.cache_size is not None:
+                cache = Cache.setdefault(i, {})
+                stats = Cache["stats"].setdefault(i, {"hit": 0, "miss": 0})
 
-        for i, di in enumerate(deltas):
-            # Eliminate those indexes where we are outside grid range or exactly on the grid point
-            if points[i] >= self.gridtuples[i][-1]:
-                cube = cube[1]
-            elif points[i] <= self.gridtuples[i][0]:
-                cube = cube[0]
-            # Otherwise eliminate index by linear interpolation
+                if point in cache:
+                    data = cache[point]
+                    stats["hit"] += 1
+                else:
+                    # Simple FIFO
+                    if self.cache_size and len(cache) >= self.cache_size:
+                        cache.pop(list(cache)[0])
+
+                    data = self._lookup(i, point)
+                    cache[point] = data
+                    stats["miss"] += 1
             else:
-                cube[0] *= diff[i]
-                cube[1] *= di
+                data = self._lookup(i, point)
+
+            deltas[i], idxs[i] = data
+
+        cube = np.copy(self.gridarrays[tuple(idxs)], order="A")
+
+        # Only linear interpolate sliced dimensions
+        for i, idx in enumerate(idxs):
+            if isinstance(idx, slice):
+                cube[0] *= 1 - deltas[i]
+                cube[1] *= deltas[i]
                 cube[0] += cube[1]
                 cube = cube[0]
 
@@ -242,11 +191,14 @@ class VectorInterpolator:
         Passes args to the appropriate interpolation method defined by the version at
         object init.
         """
-        if self.method == 1:
+        if self.method == -1:
+            return self.value
+        elif self.method == 1:
             return self._interpolate(*args, **kwargs)
         elif self.method == 2:
             return self._multilinear_grid(*args, **kwargs)
 
+
 def load_wavelen(wavelength_file: str):
     """Load a wavelength file, and convert to nanometers if needed.
 
@@ -267,7 +219,9 @@ def load_wavelen(wavelength_file: str):
     return wl, fwhm
 
 
-def emissive_radiance(emissivity: np.array, T: np.array, wl: np.array) -> (np.array, np.array):
+def emissive_radiance(
+    emissivity: np.array, T: np.array, wl: np.array
+) -> (np.array, np.array):
     """Calcluate the radiance of a surface due to emission.
 
     Args:
@@ -281,18 +235,27 @@ def emissive_radiance(emissivity: np.array, T: np.array, wl: np.array) -> (np.ar
 
     """
 
-    c_1 = 1.88365e32/np.pi
+    c_1 = 1.88365e32 / np.pi
     c_2 = 14387690
     J_per_eV = 1.60218e-19
     wl_um = wl / 1000.0
-    ph_per_sec_cm2_sr_nm = c_1/(wl**4)/(np.exp(c_2/wl/T)-1.0) * emissivity
+    ph_per_sec_cm2_sr_nm = c_1 / (wl**4) / (np.exp(c_2 / wl / T) - 1.0) * emissivity
     # photon energy in eV
-    eV_per_sec_cm2_sr_nm = 1.2398 * ph_per_sec_cm2_sr_nm/wl_um
+    eV_per_sec_cm2_sr_nm = 1.2398 * ph_per_sec_cm2_sr_nm / wl_um
     W_per_cm2_sr_nm = J_per_eV * eV_per_sec_cm2_sr_nm
-    uW_per_cm2_sr_nm = W_per_cm2_sr_nm*1e6
-    dRdn_dT = c_1/(wl**4)*(-pow(np.exp(c_2/wl/T)-1.0, -2.0)) *\
-        np.exp(c_2/wl/T)*(-pow(T, -2)*c_2/wl) *\
-        emissivity/wl_um*1.2398*J_per_eV*1e6
+    uW_per_cm2_sr_nm = W_per_cm2_sr_nm * 1e6
+    dRdn_dT = (
+        c_1
+        / (wl**4)
+        * (-pow(np.exp(c_2 / wl / T) - 1.0, -2.0))
+        * np.exp(c_2 / wl / T)
+        * (-pow(T, -2) * c_2 / wl)
+        * emissivity
+        / wl_um
+        * 1.2398
+        * J_per_eV
+        * 1e6
+    )
     return uW_per_cm2_sr_nm, dRdn_dT
 
 
@@ -312,7 +275,9 @@ def svd_inv(C: np.array, hashtable: OrderedDict = None, max_hash_size: int = Non
     return svd_inv_sqrt(C, hashtable, max_hash_size)[0]
 
 
-def svd_inv_sqrt(C: np.array, hashtable: OrderedDict = None, max_hash_size: int = None) -> (np.array, np.array):
+def svd_inv_sqrt(
+    C: np.array, hashtable: OrderedDict = None, max_hash_size: int = None
+) -> (np.array, np.array):
     """Matrix inversion, based on decomposition.  Built to be stable, and positive.
 
     Args:
@@ -329,8 +294,8 @@ def svd_inv_sqrt(C: np.array, hashtable: OrderedDict = None, max_hash_size: int
     h = None
     if hashtable is not None:
         # If arrays are in Fortran ordering, they are not hashable.
-        if not C.flags['C_CONTIGUOUS']:
-            C = C.copy(order='C')
+        if not C.flags["C_CONTIGUOUS"]:
+            C = C.copy(order="C")
         h = xxhash.xxh64_digest(C)
         if h in hashtable:
             return hashtable[h]
@@ -338,20 +303,21 @@ def svd_inv_sqrt(C: np.array, hashtable: OrderedDict = None, max_hash_size: int
     D, P = scipy.linalg.eigh(C)
     for count in range(3):
         if np.any(D < 0) or np.any(np.isnan(D)):
-            inv_eps = 1e-6 * (count-1)*10
-            D, P = scipy.linalg.eigh(
-                C + np.diag(np.ones(C.shape[0]) * inv_eps))
+            inv_eps = 1e-6 * (count - 1) * 10
+            D, P = scipy.linalg.eigh(C + np.diag(np.ones(C.shape[0]) * inv_eps))
         else:
             break
 
         if count == 2:
-            raise ValueError('Matrix inversion contains negative values,' +
-                             'even after adding {} to the diagonal.'.format(inv_eps))
+            raise ValueError(
+                "Matrix inversion contains negative values,"
+                + "even after adding {} to the diagonal.".format(inv_eps)
+            )
 
-    Ds = np.diag(1/np.sqrt(D))
-    L = P@Ds
-    Cinv_sqrt = L@P.T
-    Cinv = L@L.T
+    Ds = np.diag(1 / np.sqrt(D))
+    L = P @ Ds
+    Cinv_sqrt = L @ P.T
+    Cinv = L @ L.T
 
     # If there is a hash table, cache our solution.  Bound the total cache
     # size by removing any extra items in FIFO order.
@@ -375,7 +341,7 @@ def expand_path(directory: str, subpath: str) -> str:
 
     """
 
-    if subpath.startswith('/'):
+    if subpath.startswith("/"):
         return subpath
     return os.path.join(directory, subpath)
 
@@ -415,15 +381,15 @@ def get_absorption(wl: np.array, absfile: str) -> (np.array, np.array):
     """
 
     # read the indices of refraction
-    q = np.loadtxt(absfile, delimiter=',')
+    q = np.loadtxt(absfile, delimiter=",")
     wl_orig_nm = q[:, 0]
-    wl_orig_cm = wl_orig_nm/1e9*1e2
+    wl_orig_cm = wl_orig_nm / 1e9 * 1e2
     water_imag = q[:, 2]
     ice_imag = q[:, 4]
 
     # calculate absorption coefficients in cm^-1
-    water_abscf = water_imag*np.pi*4.0/wl_orig_cm
-    ice_abscf = ice_imag*np.pi*4.0/wl_orig_cm
+    water_abscf = water_imag * np.pi * 4.0 / wl_orig_cm
+    ice_abscf = ice_imag * np.pi * 4.0 / wl_orig_cm
 
     # interpolate to new wavelengths (user provides nm)
     water_abscf_intrp = np.interp(wl, wl_orig_nm, water_abscf)
@@ -505,7 +471,7 @@ def json_load_ascii(filename: str, shell_replace: bool = True) -> dict:
 
     """
 
-    with open(filename, 'r') as fin:
+    with open(filename, "r") as fin:
         j = json.load(fin)
         return recursive_reencode(j, shell_replace)
 
@@ -526,9 +492,11 @@ def expand_all_paths(to_expand: dict, absdir: str):
     def recursive_expand(j):
         if isinstance(j, dict):
             for key, value in j.items():
-                if isinstance(key, str) and \
-                    ('file' in key or 'directory' in key or 'path' in key) and \
-                        isinstance(value, str):
+                if (
+                    isinstance(key, str)
+                    and ("file" in key or "directory" in key or "path" in key)
+                    and isinstance(value, str)
+                ):
                     j[key] = expand_path(absdir, value)
                 else:
                     j[key] = recursive_expand(value)
@@ -555,18 +523,20 @@ def find_header(imgfile: str) -> str:
 
     """
 
-    if os.path.exists(imgfile+'.hdr'):
-        return imgfile+'.hdr'
-    ind = imgfile.rfind('.raw')
+    if os.path.exists(imgfile + ".hdr"):
+        return imgfile + ".hdr"
+    ind = imgfile.rfind(".raw")
     if ind >= 0:
-        return imgfile[0:ind]+'.hdr'
-    ind = imgfile.rfind('.img')
+        return imgfile[0:ind] + ".hdr"
+    ind = imgfile.rfind(".img")
     if ind >= 0:
-        return imgfile[0:ind]+'.hdr'
-    raise IOError('No header found for file {0}'.format(imgfile))
+        return imgfile[0:ind] + ".hdr"
+    raise IOError("No header found for file {0}".format(imgfile))
 
 
-def resample_spectrum(x: np.array, wl: np.array, wl2: np.array, fwhm2: np.array, fill: bool = False) -> np.array:
+def resample_spectrum(
+    x: np.array, wl: np.array, wl2: np.array, fwhm2: np.array, fill: bool = False
+) -> np.array:
     """Resample a spectrum to a new wavelength / FWHM.
        Assumes Gaussian SRFs.
 
@@ -581,19 +551,39 @@ def resample_spectrum(x: np.array, wl: np.array, wl2: np.array, fwhm2: np.array,
         np.array: interpolated radiance vector
 
     """
+    H = np.array(
+        [
+            spectral_response_function(wl, wi, fwhmi / 2.355)
+            for wi, fwhmi in zip(wl2, fwhm2)
+        ]
+    )
+    H[np.isnan(H)] = 0
 
-    H = np.array([spectral_response_function(wl, wi, fwhmi / 2.355)
-                  for wi, fwhmi in zip(wl2, fwhm2)])
-    if fill is False:
-        return np.dot(H, x[:, np.newaxis]).ravel()
-    else:
-        xnew = np.dot(H, x[:, np.newaxis]).ravel()
+    dims = len(x.shape)
+    if fill:
+        if dims > 1:
+            raise Exception("resample_spectrum(fill=True) only works with vectors")
+
+        x = x.reshape(-1, 1)
+        xnew = np.dot(H, x).ravel()
         good = np.isfinite(xnew)
         for i, xi in enumerate(xnew):
             if not good[i]:
-                nearest_good_ind = np.argmin(abs(wl2[good]-wl2[i]))
+                nearest_good_ind = np.argmin(abs(wl2[good] - wl2[i]))
                 xnew[i] = xnew[nearest_good_ind]
         return xnew
+    else:
+        # Replace NaNs with zeros
+        x[np.isnan(x)] = 0
+
+        # Matrix
+        if dims > 1:
+            return np.dot(H, x.T).T
+
+        # Vector
+        else:
+            x = x.reshape(-1, 1)
+            return np.dot(H, x).ravel()
 
 
 def load_spectrum(spectrum_file: str) -> (np.array, np.array):
@@ -633,9 +623,9 @@ def spectral_response_function(response_range: np.array, mu: float, sigma: float
 
     """
 
-    u = (response_range-mu)/abs(sigma)
-    y = (1.0/(np.sqrt(2.0*np.pi)*abs(sigma)))*np.exp(-u*u/2.0)
-    srf = y/y.sum()
+    u = (response_range - mu) / abs(sigma)
+    y = (1.0 / (np.sqrt(2.0 * np.pi) * abs(sigma))) * np.exp(-u * u / 2.0)
+    srf = y / y.sum()
     return srf
 
 
@@ -661,8 +651,9 @@ def combos(inds: List[List[float]]) -> np.array:
     return gridded_combinations
 
 
-def conditional_gaussian(mu: np.array, C: np.array, window: np.array, remain: np.array, x: np.array) -> \
-        (np.array, np.array):
+def conditional_gaussian(
+    mu: np.array, C: np.array, window: np.array, remain: np.array, x: np.array
+) -> (np.array, np.array):
     """Define the conditional Gaussian distribution for convenience.
 
     len(window)+len(remain)=len(x)
@@ -686,7 +677,7 @@ def conditional_gaussian(mu: np.array, C: np.array, window: np.array, remain: np
     C22 = C[w, w.T]
 
     Cinv = svd_inv(C11)
-    conditional_mean = mu[window] + C21 @ Cinv @ (x-mu[remain])
+    conditional_mean = mu[window] + C21 @ Cinv @ (x - mu[remain])
     conditional_cov = C22 - C21 @ Cinv @ C12
     return conditional_mean, conditional_cov
 
@@ -700,17 +691,123 @@ def envi_header(inputpath):
         str: the header file associated with the input reference.
 
     """
-    if os.path.splitext(inputpath)[-1] == '.img' or os.path.splitext(inputpath)[-1] == '.dat' or os.path.splitext(
-            inputpath)[-1] == '.raw':
+    if (
+        os.path.splitext(inputpath)[-1] == ".img"
+        or os.path.splitext(inputpath)[-1] == ".dat"
+        or os.path.splitext(inputpath)[-1] == ".raw"
+    ):
         # headers could be at either filename.img.hdr or filename.hdr.  Check both, return the one that exists if it
         # does, if not return the latter (new file creation presumed).
-        hdrfile = os.path.splitext(inputpath)[0] + '.hdr'
+        hdrfile = os.path.splitext(inputpath)[0] + ".hdr"
         if os.path.isfile(hdrfile):
             return hdrfile
-        elif os.path.isfile(inputpath + '.hdr'):
-            return inputpath + '.hdr'
+        elif os.path.isfile(inputpath + ".hdr"):
+            return inputpath + ".hdr"
         return hdrfile
-    elif os.path.splitext(inputpath)[-1] == '.hdr':
+    elif os.path.splitext(inputpath)[-1] == ".hdr":
         return inputpath
     else:
-        return inputpath + '.hdr'
+        return inputpath + ".hdr"
+
+
+def ray_start(num_cores, num_cpus=2, memory_b=-1):
+    import subprocess
+
+    base_args = [
+        "ray",
+        "start",
+        "--head",
+        "--num-cpus",
+        str(int(num_cores / num_cpus)),
+        "--include-dashboard",
+        "0",
+    ]
+    if memory_b != -1:
+        base_args.append("--memory")
+        base_args.append(str(int(memory_b / num_cpus)))
+
+    head_args = base_args.copy()
+    head_args.append("--head")
+    result = subprocess.run(head_args, capture_output=True)
+    stdout = str(result.stdout, encoding="utf-8")
+
+    if num_cpus > 1:
+        key = "--address="
+        start_loc = stdout.find(key) + len(key) + 1
+        end_loc = stdout.find("'", start_loc)
+
+        address = stdout[start_loc:end_loc]
+        base_args.append("--address")
+        base_args.append(address)
+
+        result = subprocess.run(base_args, capture_output=True)
+
+
+from datetime import datetime as dtt
+
+
+class Track:
+    """
+    Tracks and reports the percentage complete for some arbitrary sized iterable.
+
+    Borrowed from mlky
+    """
+
+    def __init__(self, total, step=5, print=print, reverse=False, message="complete"):
+        """
+        Parameters
+        ----------
+        total: int, iterable
+            Total items in iterable. If iterable, will call len() on it
+        step: float, default=0.05
+            Percentage step size to use for reporting, eg. 0.05 is every 5%
+        print: func, default=print
+            Print function to use, eg. logging.info
+        reverse: bool, default=False
+            Reverse the count such that 0 is 100%
+        message: str, default="complete"
+            Message to be included in the output
+        """
+        if hasattr(total, "__iter__"):
+            total = len(total)
+
+        self.step = step
+        self.total = total
+        self.print = print
+        self.start = dtt.now()
+        self.percent = step
+        self.reverse = reverse
+        self.message = message
+
+    def __call__(self, count):
+        """
+        Parameters
+        ----------
+        count: int, iterable
+            The current count of items finished. If iterable, will call len() on it
+
+        Returns
+        -------
+        bool
+            True if a percentage step was just crossed, False otherwise
+        """
+        if hasattr(count, "__iter__"):
+            count = len(count)
+
+        current = count / self.total
+        if self.reverse:
+            current = 1 - current
+        current *= 100
+
+        if current >= self.percent:
+            elap = dtt.now() - self.start
+            rate = elap / self.total
+            esti = 100 / self.percent * elap - elap
+
+            self.print(
+                f"{current:6.2f}% {self.message} (elapsed: {elap}, rate: {rate}, eta: {esti})"
+            )
+            self.percent += self.step
+
+            return True
+        return False
