@@ -17,49 +17,50 @@
 # ISOFIT: Imaging Spectrometer Optimal FITting
 # Author: David R Thompson, david.r.thompson@jpl.nasa.gov
 #
+from __future__ import annotations
+
+import logging
+from copy import deepcopy
 
 import numpy as np
-from copy import deepcopy
-from scipy.linalg import det, norm, pinv, sqrtm, inv, block_diag
-from importlib import import_module
 from scipy.interpolate import interp1d
 from scipy.io import loadmat
+from scipy.linalg import block_diag
 
-from .common import recursive_replace, eps
-from .instrument import Instrument
-from ..radiative_transfer.radiative_transfer import RadiativeTransfer
-from isofit.configs import Config
-
-from isofit.surface import Surface, ThermalSurface, MultiComponentSurface, GlintSurface, LUTSurface
+from isofit.core.common import eps
+from isofit.core.instrument import Instrument
+from isofit.radiative_transfer.radiative_transfer import RadiativeTransfer
+from isofit.surface import Surface
 
+Logger = logging.getLogger(__file__)
 
 ### Classes ###
 
+
 class ForwardModel:
     """ForwardModel contains all the information about how to calculate
-     radiance measurements at a specific spectral calibration, given a 
-     state vector. It also manages the distributions of unretrieved, 
-     unknown parameters of the state vector (i.e. the S_b and K_b 
-     matrices of Rodgers et al.
+    radiance measurements at a specific spectral calibration, given a
+    state vector. It also manages the distributions of unretrieved,
+    unknown parameters of the state vector (i.e. the S_b and K_b
+    matrices of Rodgers et al.
 
-     State vector elements always go in the following order: 
-       (1) Surface parameters
-       (2) Radiative Transfer (RT) parameters
-       (3) Instrument parameters
+    State vector elements always go in the following order:
+      (1) Surface parameters
+      (2) Radiative Transfer (RT) parameters
+      (3) Instrument parameters
 
-     The parameter bounds, scales, initial values, and names are all
-     ordered in this way.  The variable self.statevec contains the name
-     of each state vector element, in the proper ordering.
+    The parameter bounds, scales, initial values, and names are all
+    ordered in this way.  The variable self.statevec contains the name
+    of each state vector element, in the proper ordering.
 
-     The "b" vector corresponds to the K_b calculations in Rogers (2000); 
-     the variables bvec and bval represent the model unknowns' names and 
-     their  magnitudes, respectively.  Larger magnitudes correspond to
-     a larger variance in the unknown values.  This acts as additional 
-     noise for the purpose of weighting the measurement information
-     against the prior. """
+    The "b" vector corresponds to the K_b calculations in Rogers (2000);
+    the variables bvec and bval represent the model unknowns' names and
+    their  magnitudes, respectively.  Larger magnitudes correspond to
+    a larger variance in the unknown values.  This acts as additional
+    noise for the purpose of weighting the measurement information
+    against the prior."""
 
     def __init__(self, full_config: Config):
-
         # load in the full config (in case of inter-module dependencies) and
         # then designate the current config
         self.full_config = full_config
@@ -72,23 +73,17 @@ class ForwardModel:
         # Build the radiative transfer model
         self.RT = RadiativeTransfer(self.full_config)
 
-        # Build the surface model - this is a bit ugly with the conditionals, but the surface config should already be
-        # forced to have appropriate properties, so at least safe
-        # TODO: make surface a class with inheritance to make this cleaner
-        self.surface = None
-        if self.config.surface.surface_category == 'surface':
-            self.surface = Surface(self.full_config)
-        elif self.config.surface.surface_category == 'multicomponent_surface':
-            self.surface = MultiComponentSurface(self.full_config)
-        elif self.config.surface.surface_category == 'glint_surface':
-            self.surface = GlintSurface(self.full_config)
-        elif self.config.surface.surface_category == 'thermal_surface':
-            self.surface = ThermalSurface(self.full_config)
-        elif self.config.surface.surface_category == 'lut_surface':
-            self.surface = LUTSurface(self.full_config)
-        else:
-            raise ValueError('Must specify a valid surface model')
-            # No need to be more specific - should have been checked in config already
+        # Build the surface model
+        self.surface = Surface(full_config)
+
+        if self.surface.n_wl != len(self.RT.wl) or not np.all(
+            np.isclose(self.surface.wl, self.RT.wl, atol=0.01)
+        ):
+            Logger.warning(
+                "Surface and RTM wavelengths differ - if running at higher RTM"
+                " spectral resolution or with variable wavelength position, this"
+                " is expected.  Otherwise, consider checking the surface model."
+            )
 
         # Build combined vectors from surface, RT, and instrument
         bounds, scale, init, statevec, bvec, bval = ([] for i in range(6))
@@ -114,19 +109,39 @@ class ForwardModel:
 
         # Set up indices for references - MUST MATCH ORDER FROM ABOVE ASSIGNMENT
         self.idx_surface = np.arange(len(self.surface.statevec_names), dtype=int)
-        self.idx_RT = np.arange(len(self.RT.statevec_names), dtype=int) + len(self.idx_surface)
-        self.idx_instrument = np.arange(
-            len(self.instrument.statevec_names), dtype=int) + len(self.idx_surface) + len(self.idx_RT)
+        # Sometimes, it's convenient to have the index of the entire surface
+        # as one variable, and sometimes you want the sub-components
+        # Split surface state vector indices to cover cases where we retrieve
+        # additional non-reflectance surface parameters
+        self.idx_surf_rfl = self.idx_surface[
+            : len(self.surface.idx_lamb)
+        ]  # reflectance portion
+        self.idx_surf_nonrfl = self.idx_surface[
+            len(self.surface.idx_lamb) :
+        ]  # all non-reflectance surface parameters
+        self.idx_RT = np.arange(len(self.RT.statevec_names), dtype=int) + len(
+            self.idx_surface
+        )
+        self.idx_instrument = (
+            np.arange(len(self.instrument.statevec_names), dtype=int)
+            + len(self.idx_surface)
+            + len(self.idx_RT)
+        )
 
         self.surface_b_inds = np.arange(len(self.surface.bvec), dtype=int)
-        self.RT_b_inds = np.arange(len(self.RT.bvec), dtype=int) + len(self.surface_b_inds)
-        self.instrument_b_inds = np.arange(
-            len(self.instrument.bvec), dtype=int) + len(self.surface_b_inds) + len(self.RT_b_inds)
+        self.RT_b_inds = np.arange(len(self.RT.bvec), dtype=int) + len(
+            self.surface_b_inds
+        )
+        self.instrument_b_inds = (
+            np.arange(len(self.instrument.bvec), dtype=int)
+            + len(self.surface_b_inds)
+            + len(self.RT_b_inds)
+        )
 
         # Load model discrepancy correction
         if self.config.model_discrepancy_file is not None:
             D = loadmat(self.config.model_discrepancy_file)
-            self.model_discrepancy = D['cov']
+            self.model_discrepancy = D["cov"]
         else:
             self.model_discrepancy = None
 
@@ -134,15 +149,15 @@ class ForwardModel:
         """Check if state vector is within bounds."""
 
         x_RT = x[self.idx_RT]
-        x_surface = x[self.idx_surface]
         bound_lwr = self.bounds[0]
         bound_upr = self.bounds[1]
-        return any(x_RT >= (bound_upr[self.idx_RT] - eps*2.0)) or \
-            any(x_RT <= (bound_lwr[self.idx_RT] + eps*2.0))
+        return any(x_RT >= (bound_upr[self.idx_RT] - eps * 2.0)) or any(
+            x_RT <= (bound_lwr[self.idx_RT] + eps * 2.0)
+        )
 
     def xa(self, x, geom):
         """Calculate the prior mean of the state vector (the concatenation
-        of state vectors for the surface, Radiative Transfer model, and 
+        of state vectors for the surface, Radiative Transfer model, and
         instrument).
 
         NOTE: the surface prior mean depends on the current state;
@@ -156,8 +171,8 @@ class ForwardModel:
         return np.concatenate((xa_surface, xa_RT, xa_instrument), axis=0)
 
     def Sa(self, x, geom):
-        """Calculate the prior covariance of the state vector (the 
-        concatenation of state vectors for the surface and radiative transfer 
+        """Calculate the prior covariance of the state vector (the
+        concatenation of state vectors for the surface and radiative transfer
         model).
 
         NOTE: the surface prior depends on the current state; this
@@ -168,16 +183,15 @@ class ForwardModel:
         Sa_surface = self.surface.Sa(x_surface, geom)[:, :]
         Sa_RT = self.RT.Sa()[:, :]
         Sa_instrument = self.instrument.Sa()[:, :]
-        
+
         return block_diag(Sa_surface, Sa_RT, Sa_instrument)
 
     def calc_rdn(self, x, geom, rfl=None, Ls=None):
         """Calculate the high-resolution radiance, permitting overrides.
-        Project to top-of-atmosphere and translate to radiance. The 
-        radiative transfer calculations may take place at higher resolution 
-        so we upsample surface terms.       
+        Project to top-of-atmosphere and translate to radiance. The
+        radiative transfer calculations may take place at higher resolution
+        so we upsample surface terms.
         """
-
         x_surface, x_RT, x_instrument = self.unpack(x)
         if rfl is None:
             rfl = self.surface.calc_rfl(x_surface, geom)
@@ -186,7 +200,7 @@ class ForwardModel:
 
         rfl_hi = self.upsample(self.surface.wl, rfl)
         Ls_hi = self.upsample(self.surface.wl, Ls)
-        return self.RT.calc_rdn(x_RT, rfl_hi, Ls_hi, geom)
+        return self.RT.calc_rdn(x_RT, x_surface, rfl_hi, Ls_hi, geom)
 
     def calc_meas(self, x, geom, rfl=None, Ls=None):
         """Calculate the model observation at instrument wavelengths."""
@@ -212,24 +226,24 @@ class ForwardModel:
 
     def Seps(self, x, meas, geom):
         """Calculate the total uncertainty of the observation, including
-        up to three terms: (1) the instrument noise; (2) the uncertainty 
+        up to three terms: (1) the instrument noise; (2) the uncertainty
         due to explicit unmodeled variables, i.e. the S_epsilon matrix of
         Rodgers et al.; and (3) an aggregate 'model discrepancy' term,
         Gamma."""
 
         if self.model_discrepancy is not None:
-            Gamma = self.model_discrepancy 
+            Gamma = self.model_discrepancy
         else:
             Gamma = 0
 
         Kb = self.Kb(x, geom)
         Sy = self.instrument.Sy(meas, geom)
 
-        return Sy + Kb.dot(self.Sb).dot(Kb.T) + Gamma 
+        return Sy + Kb.dot(self.Sb).dot(Kb.T) + Gamma
 
     def K(self, x, geom):
-        """Derivative of observation with respect to state vector. This is 
-        the concatenation of jacobians with respect to parameters of the 
+        """Derivative of observation with respect to state vector. This is
+        the concatenation of jacobians with respect to parameters of the
         surface and radiative transfer model."""
 
         # Unpack state vector
@@ -248,18 +262,19 @@ class ForwardModel:
         dLs_dsurface_hi = self.upsample(self.surface.wl, dLs_dsurface.T).T
 
         # Derivatives of RTM radiance
-        drdn_dRT, drdn_dsurface = \
-            self.RT.drdn_dRT(x_RT, x_surface, rfl_hi, drfl_dsurface_hi, Ls_hi,
-                             dLs_dsurface_hi, geom)
+        drdn_dRT, drdn_dsurface = self.RT.drdn_dRT(
+            x_RT, x_surface, rfl_hi, drfl_dsurface_hi, Ls_hi, dLs_dsurface_hi, geom
+        )
 
         # Derivatives of measurement, avoiding recalculation of rfl, Ls
-        dmeas_dsurface = \
-            self.instrument.sample(x_instrument, self.RT.wl, drdn_dsurface.T).T
-        dmeas_dRT = self.instrument.sample(
-            x_instrument, self.RT.wl, drdn_dRT.T).T
+        dmeas_dsurface = self.instrument.sample(
+            x_instrument, self.RT.wl, drdn_dsurface.T
+        ).T
+        dmeas_dRT = self.instrument.sample(x_instrument, self.RT.wl, drdn_dRT.T).T
         rdn_hi = self.calc_rdn(x, geom, rfl=rfl, Ls=Ls)
-        dmeas_dinstrument = \
-            self.instrument.dmeas_dinstrument(x_instrument, self.RT.wl, rdn_hi)
+        dmeas_dinstrument = self.instrument.dmeas_dinstrument(
+            x_instrument, self.RT.wl, rdn_hi
+        )
 
         # Put it all together
         K = np.zeros((self.n_meas, self.nstate), dtype=float)
@@ -270,9 +285,9 @@ class ForwardModel:
 
     def Kb(self, x, geom):
         """Derivative of measurement with respect to unmodeled & unretrieved
-        unknown variables, e.g. S_b. This is  the concatenation of Jacobians 
-        with respect to parameters of the surface, radiative transfer model, 
-        and instrument.  Currently we only treat uncertainties in the 
+        unknown variables, e.g. S_b. This is  the concatenation of Jacobians
+        with respect to parameters of the surface, radiative transfer model,
+        and instrument.  Currently we only treat uncertainties in the
         instrument and RT model."""
 
         # Unpack state vector
@@ -285,11 +300,11 @@ class ForwardModel:
         Ls_hi = self.upsample(self.surface.wl, Ls)
         rdn_hi = self.calc_rdn(x, geom, rfl=rfl, Ls=Ls)
 
-        drdn_dRTb = self.RT.drdn_dRTb(x_RT, rfl_hi, Ls_hi, geom)
-        dmeas_dRTb = self.instrument.sample(x_instrument, self.RT.wl,
-                                            drdn_dRTb.T).T
+        drdn_dRTb = self.RT.drdn_dRTb(x_RT, x_surface, rfl_hi, Ls_hi, geom)
+        dmeas_dRTb = self.instrument.sample(x_instrument, self.RT.wl, drdn_dRTb.T).T
         dmeas_dinstrumentb = self.instrument.dmeas_dinstrumentb(
-            x_instrument, self.RT.wl, rdn_hi)
+            x_instrument, self.RT.wl, rdn_hi
+        )
 
         # Put it together
         Kb = np.zeros((self.n_meas, self.nbvec), dtype=float)
@@ -301,9 +316,13 @@ class ForwardModel:
         """State vector summary."""
 
         x_surface, x_RT, x_instrument = self.unpack(x)
-        return self.surface.summarize(x_surface, geom) + \
-            ' ' + self.RT.summarize(x_RT, geom) + \
-            ' ' + self.instrument.summarize(x_instrument, geom)
+        return (
+            self.surface.summarize(x_surface, geom)
+            + " "
+            + self.RT.summarize(x_RT, geom)
+            + " "
+            + self.instrument.summarize(x_instrument, geom)
+        )
 
     def calibration(self, x):
         """Calculate measured wavelengths and fwhm."""
@@ -313,21 +332,21 @@ class ForwardModel:
 
     def upsample(self, wl, q):
         """Linear interpolation to RT wavelengths."""
+        # Only interpolate if these aren't close
+        close = len(wl) == len(self.RT.wl) and np.allclose(wl, self.RT.wl)
 
-        # In some cases, these differ only by a tiny amount,
-        # so no need to waste time interpolating
-        if (len(wl) == len(self.RT.wl)) and np.allclose(wl, self.RT.wl):
-            return q
+        # or if any dimension is the wrong size
+        interp = (np.array(q.shape) == len(self.RT.wl)).all()
 
-        if q.ndim > 1:
-            q2 = []
-            for qi in q:
-                p = interp1d(wl, qi, fill_value='extrapolate')
-                q2.append(p(self.RT.wl))
-            return np.array(q2)
-        else:
-            p = interp1d(wl, q, fill_value='extrapolate')
-            return p(self.RT.wl)
+        if not close or interp:
+            if q.ndim > 1:
+                return np.array(
+                    [interp1d(wl, qi, fill_value="extrapolate")(self.RT.wl) for qi in q]
+                )
+            else:
+                p = interp1d(wl, q, fill_value="extrapolate")
+                return p(self.RT.wl)
+        return q
 
     def unpack(self, x):
         """Unpack the state vector in appropriate index ordering."""
