@@ -19,20 +19,25 @@
 #          Philip G Brodrick, philip.brodrick@jpl.nasa.gov
 #          Adam Erickson, adam.m.erickson@nasa.gov
 #
-
-import os
 import logging
+import multiprocessing
+import os
 import time
 
-import multiprocessing
-import numpy as np
-import ray
+# Explicitly set the number of threads to be 1, so we more effectively run in parallel
+# Must be executed before importing numpy, otherwise doesn't work
+if not os.environ.get("ISOFIT_NO_SET_THREADS"):
+    os.environ["MKL_NUM_THREADS"] = "1"
+    os.environ["OMP_NUM_THREADS"] = "1"
 
-from isofit.core.forward import ForwardModel
-from isofit.inversion.inverse import Inversion
-from isofit.inversion.inverse_mcmc import MCMCInversion
-from isofit.core.fileio import IO
+import click
+import numpy as np
+
+from isofit import checkNumThreads, ray
 from isofit.configs import configs
+from isofit.core.fileio import IO
+from isofit.core.forward import ForwardModel
+from isofit.inversion import Inversion
 
 
 class Isofit:
@@ -44,16 +49,19 @@ class Isofit:
         logfile: file to write output logs to
     """
 
-    def __init__(self, config_file, level='INFO', logfile=None):
+    def __init__(self, config_file, level="INFO", logfile=None):
+        # Check the MKL/OMP env vars and raise a warning if not set properly
+        checkNumThreads()
 
-        # Explicitly set the number of threads to be 1, so we more effectively
-        # run in parallel
-        os.environ["MKL_NUM_THREADS"] = "1"
         # Set logging level
         self.loglevel = level
         self.logfile = logfile
-        logging.basicConfig(format='%(levelname)s:%(asctime)s ||| %(message)s', level=self.loglevel,
-                            filename=self.logfile, datefmt='%Y-%m-%d,%H:%M:%S')
+        logging.basicConfig(
+            format="%(levelname)s:%(asctime)s ||| %(message)s",
+            level=self.loglevel,
+            filename=self.logfile,
+            datefmt="%Y-%m-%d,%H:%M:%S",
+        )
 
         self.rows = None
         self.cols = None
@@ -64,28 +72,26 @@ class Isofit:
         self.config.get_config_errors()
 
         # Initialize ray for parallel execution
-        rayargs = {'address': self.config.implementation.ip_head,
-                   '_redis_password': self.config.implementation.redis_password,
-                   '_temp_dir': self.config.implementation.ray_temp_dir,
-                   'ignore_reinit_error': self.config.implementation.ray_ignore_reinit_error,
-                   'include_dashboard': self.config.implementation.ray_include_dashboard,
-                   'local_mode': self.config.implementation.n_cores == 1}
+        rayargs = {
+            "address": self.config.implementation.ip_head,
+            "_redis_password": self.config.implementation.redis_password,
+            "_temp_dir": self.config.implementation.ray_temp_dir,
+            "ignore_reinit_error": self.config.implementation.ray_ignore_reinit_error,
+            "include_dashboard": self.config.implementation.ray_include_dashboard,
+            "local_mode": self.config.implementation.n_cores == 1,
+        }
 
         # We can only set the num_cpus if running on a single-node
-        if self.config.implementation.ip_head is None and self.config.implementation.redis_password is None:
-            rayargs['num_cpus'] = self.config.implementation.n_cores
+        if (
+            self.config.implementation.ip_head is None
+            and self.config.implementation.redis_password is None
+        ):
+            rayargs["num_cpus"] = self.config.implementation.n_cores
 
-        if self.config.implementation.debug_mode is False:
-            ray.init(**rayargs)
+        ray.init(**rayargs)
 
         self.workers = None
 
-    def __del__(self):
-        try:
-            ray.shutdown()
-        except:
-            return
-
     def run(self, row_column=None):
         """
         Iterate over spectra, reading and writing through the IO
@@ -105,15 +111,14 @@ class Isofit:
         """
 
         logging.info("Building first forward model, will generate any necessary LUTs")
-        fm = ForwardModel(self.config)
+        self.fm = fm = ForwardModel(self.config)
         if row_column is not None:
-            ranges = row_column.split(',')
+            ranges = row_column.split(",")
             if len(ranges) == 1:
                 self.rows, self.cols = [int(ranges[0])], None
             if len(ranges) == 2:
                 row_start, row_end = ranges
-                self.rows, self.cols = range(
-                    int(row_start), int(row_end)), None
+                self.rows, self.cols = range(int(row_start), int(row_end)), None
             elif len(ranges) == 4:
                 row_start, row_end, col_start, col_end = ranges
                 self.rows = range(int(row_start), int(row_end) + 1)
@@ -124,7 +129,9 @@ class Isofit:
             self.cols = range(io.n_cols)
             del io
 
-        index_pairs = np.vstack([x.flatten(order='f') for x in np.meshgrid(self.rows, self.cols)]).T
+        index_pairs = np.vstack(
+            [x.flatten(order="f") for x in np.meshgrid(self.rows, self.cols)]
+        ).T
 
         n_iter = index_pairs.shape[0]
 
@@ -136,43 +143,58 @@ class Isofit:
         # Max out the number of workers based on the number of tasks
         n_workers = min(n_workers, n_iter)
 
-        if not self.config.implementation.debug_mode:
-            fm_id = ray.put(fm)
-
-        if self.workers is None and not self.config.implementation.debug_mode:
-            remote_worker = ray.remote(Worker)
-            self.workers = ray.util.ActorPool([remote_worker.remote(self.config, fm_id, self.loglevel, self.logfile, n,
-                                                                    n_workers)
-                                               for n in range(n_workers)])
-        elif self.config.implementation.debug_mode:
-            self.workers = Worker(self.config, fm, self.loglevel, self.logfile, 0, 1)
+        params = [
+            ray.put(obj)
+            for obj in [self.config, fm, self.loglevel, self.logfile, n_workers]
+        ]
+        self.workers = ray.util.ActorPool(
+            [Worker.remote(*params, n) for n in range(n_workers)]
+        )
 
         start_time = time.time()
-        n_tasks = min(n_workers * self.config.implementation.task_inflation_factor, n_iter)
+        n_tasks = min(
+            n_workers * self.config.implementation.task_inflation_factor, n_iter
+        )
 
-        logging.info(f'Beginning {n_iter} inversions in {n_tasks} chunks using {n_workers} cores')
+        logging.info(
+            f"Beginning {n_iter} inversions in {n_tasks} chunks using {n_workers} cores"
+        )
 
-        if not self.config.implementation.debug_mode:
-            # Divide up spectra to run into chunks
-            index_sets = np.linspace(0, n_iter, num=n_tasks, dtype=int)
-            if len(index_sets) == 1:
-                indices_to_run = [index_pairs[0:1,:]]
-            else:
-                indices_to_run = [index_pairs[index_sets[l]:index_sets[l + 1], :]
-                                  for l in range(len(index_sets) - 1)]
-
-            res = list(self.workers.map_unordered(lambda a, b: a.run_set_of_spectra.remote(b), indices_to_run))
+        # Divide up spectra to run into chunks
+        index_sets = np.linspace(0, n_iter, num=n_tasks, dtype=int)
+        if len(index_sets) == 1:
+            indices_to_run = [index_pairs[0:1, :]]
         else:
-            self.workers.run_set_of_spectra(index_pairs)
+            indices_to_run = [
+                index_pairs[index_sets[l] : index_sets[l + 1], :]
+                for l in range(len(index_sets) - 1)
+            ]
+
+        res = list(
+            self.workers.map_unordered(
+                lambda a, b: a.run_set_of_spectra.remote(b), indices_to_run
+            )
+        )
 
         total_time = time.time() - start_time
-        logging.info(f'Inversions complete.  {round(total_time,2)}s total, {round(n_iter/total_time,4)} spectra/s, '
-                     f'{round(n_iter/total_time/n_workers,4)} spectra/s/core')
+        logging.info(
+            f"Inversions complete.  {round(total_time,2)}s total,"
+            f" {round(n_iter/total_time,4)} spectra/s,"
+            f" {round(n_iter/total_time/n_workers,4)} spectra/s/core"
+        )
 
 
+@ray.remote(num_cpus=1)
 class Worker(object):
-    def __init__(self, config: configs.Config, forward_model: ForwardModel, loglevel: str, logfile: str,
-                 worker_id: int = None, total_workers: int = None):
+    def __init__(
+        self,
+        config: configs.Config,
+        forward_model: ForwardModel,
+        loglevel: str,
+        logfile: str,
+        total_workers: int = None,
+        worker_id: int = None,
+    ):
         """
         Worker class to help run a subset of spectra.
 
@@ -184,34 +206,29 @@ class Worker(object):
             total_workers: the total number of workers running, for logging reference
         """
 
-        logging.basicConfig(format='%(levelname)s:%(asctime)s ||| %(message)s', level=loglevel, filename=logfile,
-                            datefmt='%Y-%m-%d,%H:%M:%S')
+        logging.basicConfig(
+            format="%(levelname)s:%(asctime)s ||| %(message)s",
+            level=loglevel,
+            filename=logfile,
+            datefmt="%Y-%m-%d,%H:%M:%S",
+        )
         self.config = config
         self.fm = forward_model
-        # self.fm = ForwardModel(self.config)
-
-        if self.config.implementation.mode == 'mcmc_inversion':
-            self.iv = MCMCInversion(self.config, self.fm)
-        elif self.config.implementation.mode in ['inversion', 'simulation']:
-            self.iv = Inversion(self.config, self.fm)
-        else:
-            # This should never be reached due to configuration checking
-            raise AttributeError('Config implementation mode node valid')
-
+        self.iv = Inversion(self.config, self.fm)
         self.io = IO(self.config, self.fm)
 
         self.approximate_total_spectra = None
         if total_workers is not None:
-            self.approximate_total_spectra = self.io.n_cols * self.io.n_rows / total_workers
+            self.approximate_total_spectra = (
+                self.io.n_cols * self.io.n_rows / total_workers
+            )
         self.worker_id = worker_id
         self.completed_spectra = 0
 
     def run_set_of_spectra(self, indices: np.array):
-
         for index in range(0, indices.shape[0]):
-
             logging.debug("Read chunk of spectra")
-            row, col = indices[index,0], indices[index,1]
+            row, col = indices[index, 0], indices[index, 1]
 
             input_data = self.io.get_components_at_index(row, col)
 
@@ -229,20 +246,54 @@ class Worker(object):
                 try:
                     self.io.write_spectrum(row, col, states, self.fm, self.iv)
                 except ValueError as err:
-                    logging.info(
+                    logging.exception(
                         f"""
-                        Encountered the following ValueError in (row,col) ({row},{col}).
-                        Results for this pixel will be all zeros.
-                        """
+                    Encountered the following ValueError in (row,col) ({row},{col}).
+                    Results for this pixel will be all zeros.
+                    """
                     )
-                    logging.error(err)
 
                 if index % 100 == 0:
-                    if self.worker_id is not None and self.approximate_total_spectra is not None:
-                        percent = np.round(self.completed_spectra / self.approximate_total_spectra * 100,2)
-                        logging.info(f'Worker {self.worker_id} completed {self.completed_spectra}/'
-                                     f'~{self.approximate_total_spectra}:: {percent}% complete')
-                    else:
-                        logging.info(f'Worker at start location ({row},{col}) completed {index}/{indices.shape[0]}')
+                    if (
+                        self.worker_id is not None
+                        and self.approximate_total_spectra is not None
+                    ):
+                        percent = np.round(
+                            self.completed_spectra
+                            / self.approximate_total_spectra
+                            * 100,
+                            2,
+                        )
+                        logging.info(
+                            f"Worker {self.worker_id} completed"
+                            f" {self.completed_spectra}/~{self.approximate_total_spectra}::"
+                            f" {percent}% complete"
+                        )
+        logging.info(
+            f"Worker at start location ({row},{col}) completed"
+            f" {index}/{indices.shape[0]}"
+        )
 
         self.io.flush_buffers()
+
+
+@click.command(name="run")
+@click.argument("config_file")
+@click.option(
+    "-l",
+    "--level",
+    help="Log level",
+    type=click.Choice(
+        ["DEBUG", "INFO", "WARNING", "ERROR", "EXCEPTION"], case_sensitive=True
+    ),
+    default="INFO",
+)
+def cli_run(config_file, level):
+    """Execute ISOFIT core"""
+
+    click.echo(f"Running ISOFIT(config_file={config_file!r}, level={level})")
+
+    logging.basicConfig(format="%(message)s", level=level)
+    Isofit(config_file=config_file, level=level).run()
+
+    click.echo("Done")
